@@ -1,4 +1,4 @@
-from typing import Any, Dict, Generator, List, Union
+from typing import Any, Callable, Dict, Generator, List, Union
 
 from collections import defaultdict
 from datetime import datetime
@@ -9,11 +9,15 @@ import os
 import re
 import shutil
 
+import pandas as pd
 from schematics.exceptions import DataError, ValidationError
+import dask.dataframe as dd
 import jsoncomment as jsonc
 import OpenEXR as openexr
 
-from pandas import DataFrame
+FilePath = Union[str, Path]
+DF = Union[pd.DataFrame, dd.DataFrame]
+DFS = Union[pd.DataFrame, pd.Series, dd.DataFrame, dd.Series]
 # ------------------------------------------------------------------------------
 
 
@@ -23,7 +27,7 @@ The tools module contains general functions useful to other hidebound modules.
 
 
 def list_all_files(directory, include_regex='', exclude_regex=''):
-    # type: (Union[str, Path], str, str) -> Generator[Path, None, None]
+    # type: (FilePath, str, str) -> Generator[Path, None, None]
     '''
     Recusively list all files within a given directory.
 
@@ -64,7 +68,7 @@ def list_all_files(directory, include_regex='', exclude_regex=''):
 
 
 def delete_empty_directories(directory):
-    # type: (Union[str, Path]) -> None
+    # type: (FilePath) -> None
     '''
     Recurses given directory tree and deletes directories that do not contain
     files or directories trees with files. .DS_Store files do not count as
@@ -96,9 +100,9 @@ def delete_empty_directories(directory):
 
 
 def directory_to_dataframe(directory, include_regex='', exclude_regex=r'\.DS_Store'):
-    # type: (Union[str, Path], str, str) -> DataFrame
+    # type: (FilePath, str, str) -> pd.DataFrame
     r'''
-    Recursively list files with in a given directory as rows in a DataFrame.
+    Recursively list files with in a given directory as rows in a pd.DataFrame.
 
     Args:
         directory (str or Path): Directory to walk.
@@ -108,7 +112,7 @@ def directory_to_dataframe(directory, include_regex='', exclude_regex=r'\.DS_Sto
             Default: '\.DS_Store'.
 
     Returns:
-        DataFrame: DataFrame with one file per row.
+        pd.DataFrame: pd.DataFrame with one file per row.
     '''
     files = list_all_files(
         directory,
@@ -117,10 +121,11 @@ def directory_to_dataframe(directory, include_regex='', exclude_regex=r'\.DS_Sto
     )  # type: Any
     files = sorted(list(files))
 
-    data = DataFrame()
+    data = pd.DataFrame()
     data['filepath'] = files
     data['filename'] = data.filepath.apply(lambda x: x.name)
-    data['extension'] = data.filepath.apply(lambda x: os.path.splitext(x)[-1][1:])
+    data['extension'] = data.filepath \
+        .apply(lambda x: os.path.splitext(x)[-1].lstrip('.'))
     data.filepath = data.filepath.apply(lambda x: x.absolute().as_posix())
     return data
 
@@ -177,7 +182,7 @@ def to_prototype(dicts):
 
 
 def read_exr_header(fullpath):
-    # type: (Union[str, Path]) -> dict
+    # type: (FilePath) -> dict
     '''
     Reads an OpenEXR image file header.
 
@@ -209,7 +214,7 @@ def time_string():
 
 
 def write_json(data, filepath):
-    # type: (object, Union[Path, str]) -> None
+    # type: (object, FilePath) -> None
     '''
     Convenience function for writing objects to JSON files.
     Writes lists with 1 item per line.
@@ -229,7 +234,7 @@ def write_json(data, filepath):
 
 
 def read_json(filepath):
-    # type (Union[Path, str]) -> object
+    # type (FilePath) -> object
     '''
     Convenience function for reading JSON files.
     Files may include comments.
@@ -249,3 +254,122 @@ def read_json(filepath):
         msg += 'Please remove any inline comments.'
         raise json.JSONDecodeError(msg, '', 0)
     return output
+
+
+def get_meta_kwargs(data, meta):
+    # type: (DFS, Any) -> dict
+    '''
+    Convenience utility for coercing the meta keyword between pandas and dask.
+
+    Args:
+        data (DataFrame or Series): Pandas or dask object.
+        meta (object): Meta key word argument.
+
+    Returns:
+        dict: Appropriate keyword args.
+    '''
+    kwargs = {}
+    if meta != '__no_default__' and data.__class__ in [dd.DataFrame, dd.Series]:
+        kwargs = dict(meta=meta)
+    return kwargs
+
+
+def pred_combinator(
+    data,                   # type: DFS
+    predicate,              # type: Callable[[Any], bool]
+    true_func,              # type: Callable[[Any], Any]
+    false_func,             # type: Callable[[Any], Any]
+    meta='__no_default__',  # type: Any
+):
+    # type: (...) -> DFS
+    '''
+    Apply true_func to rows where predicate if true and false_func to rows where
+    it is false.
+
+    Args:
+        data (DataFrame): DataFrame or Series.
+        predicate (function): Function that expects a row and returns a bool.
+        true_func (function): Function that expects a row. Called when predicate
+            is true.
+        false_func (function): Function that expects a row. Called when predicate
+            is false.
+        meta (object, optional): Metadata inference. Default: '__no_default__'.
+
+    Returns:
+        DataFrame or Series: Apply results.
+    '''
+    kwargs = get_meta_kwargs(data, meta)
+    if data.__class__ in [pd.DataFrame, dd.DataFrame]:
+        return data.apply(
+            lambda x: true_func(x) if predicate(x) else false_func(x),
+            axis=1,
+            **kwargs,
+        )
+    return data.apply(
+        lambda x: true_func(x) if predicate(x) else false_func(x),
+        **kwargs,
+    )
+
+
+def get_lut(data, column, aggregator, meta='__no_default__'):
+    # type: (DF, str, Callable[[DF], Any], Any) -> DF
+    '''
+    Constructs a lookup table with the given column as its keys and the
+    aggregator results as its values.
+    Data is grouped by given column and the given aggregator is applied to each
+    group of values.
+
+    Args:
+        data (DataFrame): DataFrame.
+        column (str): Column to be used as the key.
+        aggregator (function): Function that expects a group DataFrame and
+            returns a scalar.
+        meta (object, optional): Metadata inference. Default: '__no_default__'.
+
+    Returns:
+        DataFrame: DataFrame with key and value columns.
+    '''
+    kwargs = get_meta_kwargs(data, ('value', meta))
+    merge = pd.merge
+    empty = pd.DataFrame(columns=['key', 'value'])
+    if isinstance(data, dd.DataFrame):
+        merge = dd.merge
+        empty = dd.from_pandas(empty, npartitions=1)
+
+    grp = data.groupby(column)
+    keys = grp[column].first().to_frame(name='key')
+    if len(keys) == 0:
+        return empty
+    vals = grp.apply(aggregator, **kwargs).to_frame(name='value')
+    lut = merge(keys, vals, left_index=True, right_index=True) \
+        .reset_index(drop=True)
+    return lut
+
+
+def lut_combinator(
+    data, key_column, value_column, aggregator, meta='__no_default__'
+):
+    # type: (DF, str, str, Callable[[DF], Any], Any) -> DF
+    '''
+    Constructs a lookup table from given key_column, then applies it to given
+    data as value column.
+
+    Args:
+        data (DataFrame): DataFrame.
+        key_column (str): Column to be used as the lut keys.
+        value_column (str): Column to be used as the values.
+        aggregator (function): Function that expects a pd.DataFrame.
+        meta (object, optional): Metadata irom_nference. Default: '__no_default__'.
+
+    Returns:
+        DataFrame: DataFrame with value column.
+    '''
+    kwargs = get_meta_kwargs(data, meta)
+    merge = pd.merge
+    if isinstance(data, dd.DataFrame):
+        merge = dd.merge
+
+    lut = get_lut(data, key_column, aggregator, **kwargs)
+    lut.columns = [key_column, value_column]
+    data = merge(data, lut, on=key_column, how='left')
+    return data
